"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_1 = __importStar(require("./request"));
const locks_1 = require("../model/locks");
const base64_1 = require("../utils/base64");
const axios = require('axios'); // eslint-disable-line
class Locks {
    constructor(configuration) {
        this.url = `${configuration.values().coordinationEndpoint}/locks`;
        this.request = new request_1.default(configuration.identify);
    }
    get(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.url}/base64:${(0, base64_1.encode)(name.name)}`;
            const result = yield this.request.send((token) => (0, request_1.seaFetch)(token).get(url));
            return (0, locks_1.toLock)(result);
        });
    }
    acquire(name, clientId, ttl) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.url}/base64:${(0, base64_1.encode)(name.name)}`;
            const params = {
                'client-id': clientId,
                ttl: String(ttl),
            };
            const data = {};
            const result = yield this.request.send((token) => (0, request_1.seaFetch)(token).post(`${url}?` + new URLSearchParams(params), "{}"));
            return (0, locks_1.toHeldLock)(result);
        });
    }
    release(name, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.url}/base64:${(0, base64_1.encode)(name.name)}`;
            const params = {
                id: id,
            };
            const result = yield this.request.send((token) => (0, request_1.seaFetch)(token).delete(`${url}?` + new URLSearchParams(params)));
            return result === 'OK';
        });
    }
    renew(name, id, ttl) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.url}/base64:${(0, base64_1.encode)(name.name)}`;
            const params = {
                id,
                ttl: String(ttl),
            };
            const result = yield this.request.send((token) => (0, request_1.seaFetch)(token).post(`${url}?` + new URLSearchParams(params), "{}"));
            return result === 'OK';
        });
    }
    getPage(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.url;
            if (options === null || options === void 0 ? void 0 : options.directory) {
                url = `${this.url}/base64:${(0, base64_1.encode)(options.directory.name)}/`;
            }
            let params = {};
            if (options === null || options === void 0 ? void 0 : options.fromLock) {
                params = {
                    from: `base64:${(0, base64_1.encode)(options.fromLock.name)}`,
                };
            }
            const result = yield this.request.send((token) => (0, request_1.seaFetch)(token).get(url));
            return (0, locks_1.toLockPage)(result);
        });
    }
    getAllPages(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const pages = [];
            let forNextLock = options === null || options === void 0 ? void 0 : options.fromLock;
            while (true) {
                const pageResult = yield this.getPage({
                    directory: options === null || options === void 0 ? void 0 : options.directory,
                    fromLock: forNextLock,
                });
                const page = pageResult;
                pages.push(...page.locks);
                if (page.nextLock) {
                    forNextLock = page.nextLock;
                }
                else {
                    return pages;
                }
            }
        });
    }
}
exports.default = Locks;
