import Request, { seaFetch } from './request';
import { toLock, toHeldLock, toLockPage } from '../model/locks';
import { encode } from '../utils/base64';
const axios = require('axios'); // eslint-disable-line
export default class Locks {
    url;
    request;
    constructor(configuration) {
        this.url = `${configuration.values().coordinationEndpoint}/locks`;
        this.request = new Request(configuration.identify);
    }
    async get(name) {
        const url = `${this.url}/base64:${encode(name.name)}`;
        const result = await this.request.send((token) => seaFetch(token).get(url));
        return toLock(result);
    }
    async acquire(name, clientId, ttl) {
        const url = `${this.url}/base64:${encode(name.name)}`;
        const params = {
            'client-id': clientId,
            ttl: String(ttl),
        };
        const data = {};
        const result = await this.request.send((token) => seaFetch(token).post(`${url}?` + new URLSearchParams(params), "{}"));
        return toHeldLock(result);
    }
    async release(name, id) {
        const url = `${this.url}/base64:${encode(name.name)}`;
        const params = {
            id: id,
        };
        const result = await this.request.send((token) => seaFetch(token).delete(`${url}?` + new URLSearchParams(params)));
        return result === 'OK';
    }
    async renew(name, id, ttl) {
        const url = `${this.url}/base64:${encode(name.name)}`;
        const params = {
            id,
            ttl: String(ttl),
        };
        const result = await this.request.send((token) => seaFetch(token).post(`${url}?` + new URLSearchParams(params), "{}"));
        return result === 'OK';
    }
    async getPage(options) {
        let url = this.url;
        if (options?.directory) {
            url = `${this.url}/base64:${encode(options.directory.name)}/`;
        }
        let params = {};
        if (options?.fromLock) {
            params = {
                from: `base64:${encode(options.fromLock.name)}`,
            };
        }
        const result = await this.request.send((token) => seaFetch(token).get(url));
        return toLockPage(result);
    }
    async getAllPages(options) {
        const pages = [];
        let forNextLock = options?.fromLock;
        while (true) {
            const pageResult = await this.getPage({
                directory: options?.directory,
                fromLock: forNextLock,
            });
            const page = pageResult;
            pages.push(...page.locks);
            if (page.nextLock) {
                forNextLock = page.nextLock;
            }
            else {
                return pages;
            }
        }
    }
}
