import { log } from '../logging';
import { HTTPError } from '../model/errors';
export const headers = (auth) => ({
    Accept: 'application/json',
    'Content-Type': 'application/json',
    Authorization: `Bearer ${auth}`,
});
export default class Request {
    identify;
    constructor(identify) {
        this.identify = identify;
    }
    async renewIfFails(error, bodyError, request) {
        const httpError = new HTTPError(error.status, bodyError);
        if (error.status != 401 || !this.identify.autoRenew) {
            throw httpError;
        }
        log.info('Auto-Renew, renewing the token...');
        const token = await this.identify.renewToken();
        return await request(token);
    }
    async send(request) {
        const accessToken = this.identify.accessToken || (await this.identify.getToken());
        const response = await request(accessToken);
        const bodyText = await response.text();
        if (response.ok) {
            return this.parse(bodyText) ?? bodyText;
        }
        else {
            log.error(`Request error: ${bodyText}`);
            return await this.renewIfFails(response, bodyText, request);
        }
    }
    parse(body) {
        try {
            return JSON.parse(body);
        }
        catch (e) {
            return undefined;
        }
    }
}
export const seaFetch = (token) => ({
    get: async (url) => (fetch(url, {
        method: 'GET',
        headers: headers(token),
    })),
    post: async (url, body) => (await fetch(url, {
        method: 'POST',
        headers: headers(token),
        body
    })),
    put: async (url, body) => (await fetch(url, {
        method: 'PUT',
        headers: headers(token),
        body
    })),
    delete: async (url) => (fetch(url, {
        method: 'DELETE',
        headers: headers(token),
    })),
    patch: async (url) => (fetch(url, {
        method: 'PATCH',
        headers: headers(token),
    })),
});
